#ifndef CEX_TEST_ASSERT_H
#define CEX_TEST_ASSERT_H

#include <cex/ansi/codes.h>
#include <stdbool.h>
#include <stddef.h>
#include <stdio.h>

#define INFO_BAR_WIDTH 50

typedef struct {
  size_t test_count;
  size_t fail_count;
} test_result_t;

void _print_assert(bool success, const char* label, const char* file, size_t line);
void _print_project_info(test_result_t final_result);

#define TEST(label)                                            \
  test_result_t test_##label() {                               \
    test_result_t result = {.test_count = 0, .fail_count = 0}; \
    fprintf(stderr, ANSI_BOLD "  " ANSI_UNDERLINE "test: %s\n" ANSI_RESET, #label);

#define END_TEST \
  return result; \
  }

#define PROJECT(label)                                               \
  int main() {                                                       \
    test_result_t result = {.test_count = 0, .fail_count = 0};       \
    test_result_t final_result = {.test_count = 0, .fail_count = 0}; \
    fprintf(stderr, ANSI_BOLD ANSI_UNDERLINE "project: %s\n" ANSI_RESET, label);

#define END_PROJECT                    \
  _print_project_info(final_result);   \
  return final_result.fail_count != 0; \
  }

#define LABEL(label) fprintf(stderr, ANSI_ITALIC "    %s\n" ANSI_RESET, label);

#define ASSERT_EQ(label, a, b)                               \
  _print_assert((a) == (b), label, __FILE_NAME__, __LINE__); \
  result.fail_count += (a) == (b) ? 0 : 1;                   \
  result.test_count += 1;

#define ASSERT_NEQ(label, a, b)                              \
  _print_assert((a) != (b), label, __FILE_NAME__, __LINE__); \
  result.fail_count += (a) != (b) ? 0 : 1;                   \
  result.test_count += 1;

#define RUN_TEST(func)                          \
  result = test_##func();                       \
  final_result.fail_count += result.fail_count; \
  final_result.test_count += result.test_count;

#endif