#include <cex/containers/linked_list.h>
#include <cex/test/assert.h>
#include <stddef.h>
#include <stdint.h>
#include <stdlib.h>

TEST(linked_list)

LABEL("create")
linked_list_t list = LL_create(sizeof(uint64_t));
ASSERT_EQ("data size set", list.data_size, sizeof(uint64_t))
ASSERT_EQ("length = 0", list.length, 0)
ASSERT_EQ("first = NULL", list.first, NULL)

LABEL("add")
int64_t num = 69;
LL_add_value(&list, &num);
ASSERT_EQ("length = 1", list.length, 1)
ASSERT_NEQ("first = NULL", list.first, NULL)
ASSERT_EQ("next = first", list.first->next, list.first)
ASSERT_EQ("prev = first", list.first->prev, list.first)
ASSERT_EQ("*data = num", *(int64_t*)list.first->data, num)

LABEL("destroy")
LL_destroy(&list);
ASSERT_EQ("length = 0", list.length, 0)
ASSERT_EQ("first = NULL", list.first, NULL)

LABEL("from array")
uint64_t array[10] = {10, 9, 8, 7, 6, 5, 4, 3, 2, 1};
list = LL_from_array(sizeof(uint64_t), array, 10);

LABEL("iterate")
LL_iterate(&list, LL_ITER_PRINT, NULL);

LABEL("to array")
uint64_t* result_array = LL_to_array(&list);
for (index_t i = 0; i < 10; i++) {
  ASSERT_EQ("=", array[i], result_array[i]);
}
free(result_array);

LL_destroy(&list);

END_TEST